@@ -8,6 +8,7 @@ import logging
 import multiprocessing
 import time
 import traceback
+import warnings
 
 import aiohttp
 import requests
@@ -25,7 +26,7 @@ log = logging.getLogger(__name__)
 def log_future_exception(fut):
     e = fut.exception()
     if e:
-        log.exception(e)
+        log.error('got unhandled exception in future', exc_info=e)
 
 
 class LeaseManager:
@@ -78,16 +79,16 @@ class LeaseManager:
                 resp.raise_for_status()
                 task = resp.json()
             except requests.exceptions.HTTPError as e:
-                log.error('failed to autorenew task: %s', task['name'])
                 try:
-                    log.error(resp.json())
+                    data = resp.json()
                 except ValueError:
-                    log.error(resp.text())
-                log.exception(e)
+                    data = resp.text()
+                log.error('failed to autorenew task %s: %s', task['name'],
+                          data, exc_info=e)
                 event.set()
             except Exception as e:  # pylint: disable=broad-except
-                log.error('failed to autorenew task: %s', task['name'])
-                log.exception(e)
+                log.error('failed to autorenew task: %s', task['name'],
+                          exc_info=e)
                 event.set()
 
         return task
@@ -102,6 +103,11 @@ class TaskManager:
                  max_concurrency=100, retry_limit=None, session=None,
                  token=None):
         # pylint: disable=too-many-arguments,too-many-locals
+        if session:
+            warnings.warn('passing in a session is deprecated and will be '
+                          'removed in gcloud-aio-taskqueue==2.0.0',
+                          DeprecationWarning)
+
         self.worker = worker
 
         self.backoff = backoff(base=backoff_base, factor=backoff_factor,
@@ -115,17 +121,11 @@ class TaskManager:
         self.manager = multiprocessing.Manager()
         self.semaphore = asyncio.BoundedSemaphore(max_concurrency)
         self.tq = TaskQueue(project, service_file, taskqueue,
-                            location=location, session=session, token=token)
+                            location=location, token=token)
 
+        self.session = None
         self.running = False
 
-    @staticmethod
-    def get_session():
-        connector = aiohttp.TCPConnector(enable_cleanup_closed=True,
-                                         force_close=True, limit_per_host=1)
-        return aiohttp.ClientSession(connector=connector, conn_timeout=10,
-                                     read_timeout=10)
-
     async def fail(self, task, payload, exception):
         if not self.deadletter_insert_function:
             return
@@ -143,21 +143,35 @@ class TaskManager:
         await self.deadletter_insert_function(task['name'], properties)
 
     async def poll(self):
+        if not self.session:
+            connector = aiohttp.TCPConnector(enable_cleanup_closed=True,
+                                             force_close=True,
+                                             limit_per_host=1)
+            self.session = aiohttp.ClientSession(connector=connector,
+                                                 conn_timeout=10,
+                                                 read_timeout=10)
+
         while self.running:
             # only lease new tasks when we have room
             async with self.semaphore:
                 task_lease = None
                 try:
-                    async with self.get_session() as session:
-                        task_lease = await self.tq.lease(
-                            lease_seconds=self.lease_seconds,
-                            num_tasks=self.batch_size, session=session)
+                    task_lease = await self.tq.lease(
+                        lease_seconds=self.lease_seconds,
+                        num_tasks=self.batch_size, session=self.session)
                 except concurrent.futures.CancelledError:
                     return
                 except concurrent.futures.TimeoutError:
                     pass
+                except aiohttp.client_exceptions.ClientResponseError as e:
+                    if e.code != 429:
+                        log.error('got error attempting to lease tasks'
+                                  ', retrying', exc_info=e)
+                        continue
+                    # otherwise, churn with backoff
                 except Exception as e:  # pylint: disable=broad-except
-                    log.exception(e)
+                    log.error('got error attempting to lease tasks, retrying',
+                              exc_info=e)
 
             if not task_lease:
                 await asyncio.sleep(next(self.backoff))
@@ -188,50 +202,48 @@ class TaskManager:
         # get an exception while creating a LeaseManager, that's enough for me
         # to be reasonably sure the pool is broken without needing proof.
         except Exception as e:  # pylint: disable=broad-except
-            log.error('process pool broke, quitting TaskManager')
-            log.exception(e)
+            log.error('process pool broke, quitting TaskManager', exc_info=e)
             self.running = False
             return
 
         try:
-            async with self.semaphore, self.get_session() as session:
+            async with self.semaphore:
                 log.info('processing task: %s', name)
 
                 try:
                     await self.worker(payload)
                 except FailFastError as e:
                     log.error('[FailFastError] failed to process task: %s',
-                              name)
-                    log.exception(e)
+                              name, exc_info=e)
 
                     if autorenew is not None:
                         task = await autorenew.stop()
-                    await self.tq.delete(name, session=session)
+                    await self.tq.delete(name, session=self.session)
                     await self.fail(task, payload, e)
                     return
                 except Exception as e:  # pylint: disable=broad-except
-                    log.error('failed to process task: %s', name)
-                    log.exception(e)
+                    log.error('failed to process task: %s', name, exc_info=e)
 
                     if autorenew is not None:
                         task = await autorenew.stop()
 
                     tries = task['status']['attemptDispatchCount']
                     if self.retry_limit is None or tries < self.retry_limit:
-                        await self.tq.cancel(task, session=session)
+                        await self.tq.cancel(task, session=self.session)
                         return
 
                     log.warning('exceeded retry_limit, failing task')
-                    await self.tq.delete(name, session=session)
+                    await self.tq.delete(name, session=self.session)
                     await self.fail(task, payload, e)
                     return
 
                 log.info('successfully processed task: %s', name)
                 if autorenew is not None:
                     task = await autorenew.stop()
-                await self.tq.ack(task, session=session)
+                await self.tq.ack(task, session=self.session)
         except Exception as e:  # pylint: disable=broad-except
-            log.exception(e)
+            log.error('got unhandled error attempting to process task %s',
+                      name, exc_info=e)
         finally:
             if autorenew is not None:
                 await autorenew.stop()
